/* ------------------------------------------------------------------------
 * $Id: Adapters.hh,v 1.2 2002/03/22 10:32:23 elm Exp $
 *
 * This file is part of 3Dwm: The Three-Dimensional User Environment.
 *
 * 3Dwm: The Three-Dimensional User Environment:
 *	<http://www.3dwm.org>
 *
 * Chalmers Medialab
 * 	<http://www.medialab.chalmers.se>
 * 
 * ------------------------------------------------------------------------
 * File created 2001-12-05 by Niklas Elmqvist.
 *
 * Copyright (c) 2001 Niklas Elmqvist <elm@3dwm.org>.
 * ------------------------------------------------------------------------
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 * ------------------------------------------------------------------------
 */

#ifndef _Adapters_hh_
#define _Adapters_hh_

// -- Template Definitions

/**
 * Indexed adapter template. Adapts between different types of indexed
 * containers so that Celsius can work with them.
 **/ 
template <class S, class E>
class IndexedAdapter {
public:
    IndexedAdapter(const S &impl) : _impl(impl) { }
    const E & operator [] (int n) const { return _impl[n]; }
    size_t length() const { return _impl.size(); }
private:
    const S &_impl;
};

/**
 * 3D vertex adapter template. Adapts between different types of 3D
 * vertices so that Celsius can work with them. 
 **/ 
template <class V>
class VertexAdapter {
public:
    VertexAdapter(const V &v) : _v(v) { }
    float x() const { return _v.x(); }
    float y() const { return _v.y(); }
    float z() const { return _v.z(); }
public:
    const V &_v;
};

/*
template <class M> 
class MatrixAdapter {
public:
    MatrixAdapter(const M &m) : _m(m) { }
    
};
*/

#endif /* Adapters.hh */
