/* ------------------------------------------------------------------------
 * $Id: UpdateTraversal.hh,v 1.2 2002/03/22 10:32:23 elm Exp $
 *
 * This file is part of 3Dwm: The Three-Dimensional User Environment.
 *
 * 3Dwm: The Three-Dimensional User Environment:
 *	<http://www.3dwm.org>
 *
 * Chalmers Medialab
 * 	<http://www.medialab.chalmers.se>
 * 
 * ------------------------------------------------------------------------
 * File created 2002-03-20 by Niklas Elmqvist.
 *
 * Copyright (c) 2002 Niklas Elmqvist <elm@3dwm.org>.
 * ------------------------------------------------------------------------
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 * ------------------------------------------------------------------------
 */

#ifndef _UpdateTraversal_hh_
#define _UpdateTraversal_hh_

// -- 3Dwm Includes
#include "Nobel/CORBA.hh"
#include "Nobel/Visitor.hh"

// -- Forward Declarations
class NodeImpl;

// -- Class Declarations

/**
 * Scene graph update traversal class. This traversal will walk the
 * scene graph once before every rendering update, performing various
 * updates on the scene graph nodes.
 **/
class UpdateTraversal : public virtual POA_Nobel::Visitor,
			public virtual PortableServer::RefCountServantBase {
public:
    
    /**
     * Constructor.
     **/
    UpdateTraversal();

    /**
     * Destructor.
     **/
    virtual ~UpdateTraversal();

    /**
     * Perform the update traversal on a scene graph rooted at a given
     * node.
     *
     * @param root scene graph root node.
     **/
    void update(NodeImpl *root);

    /// IDL Functions
    virtual void visit(Nobel::Node_ptr n);
    virtual void traverse(Nobel::Node_ptr child, Nobel::Transform_ptr t);
    virtual Nobel::Visitor::TraversalType getType() {
	return Nobel::Visitor::PostOrder;
    }
    
private:
    typedef enum { no = 0, yes, maybe } tristate_t;
    tristate_t _bv_need_update;
};

#endif /* UpdateTraversal.hh */
