/* ------------------------------------------------------------------------
 * $Id: VolumeImpl.hh,v 1.6 2002/12/07 16:27:18 steve Exp $
 *
 * This file is part of 3Dwm: The Three-Dimensional User Environment.
 *
 * 3Dwm: The Three-Dimensional User Environment:
 *	<http://www.3dwm.org>
 *
 * Chalmers Medialab
 * 	<http://www.medialab.chalmers.se>
 * 
 * ------------------------------------------------------------------------
 * File created 2001-05-29 by Niklas Elmqvist.
 *
 * Copyright (c) 2001 Niklas Elmqvist <elm@3dwm.org>.
 * ------------------------------------------------------------------------
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 * ------------------------------------------------------------------------
 */

#ifndef _VolumeImpl_hh_
#define _VolumeImpl_hh_

// -- 3Dwm Includes
#include "Nobel/CORBA.hh"
#include "Nobel/Volume.hh"
#include "Nobel/Renderer.hh"
#include "Celsius/BoundingBox.hh"

// -- Class Declarations

/**
 * C++ implementation of the Volume interface.
 **/ 
class VolumeImpl : public virtual POA_Nobel::Volume,
		   public virtual PortableServer::RefCountServantBase {
    
public:

    /**
     * Constructor.
     **/
    VolumeImpl();

    /**
     * Copy constructor.
     **/
    VolumeImpl(const VolumeImpl &v);

    /**
     * Destructor.
     **/
    virtual ~VolumeImpl();
    
    // IDL functions
    virtual CORBA::Boolean isInside(const Nobel::Vertex3D &v);
    virtual void mergeWithBox(const Nobel::Volume::Box &b);
    //virtual Nobel::Vertex3D getCenter();
    //virtual Nobel::Vertex3D getExtents();
    //virtual void getTransformationMatrix(Nobel::Matrix m);
    virtual Nobel::Volume::Box getBox();

    virtual void render(Nobel::Renderer_ptr r);
    
    /**
     * Update the volume with new vertex data.
     *
     * @param vertices sequence of vertices to use as basis for
     *                 update.
     **/ 
    virtual void update(const Nobel::Vertex3DSeq &vertices);
    
    /**
     * Equality operator.
     **/
    virtual bool operator == (const VolumeImpl &v) const;
	virtual bool operator != (const VolumeImpl &v) const;
        
private:
    BoundingBox _box;
};

// -- Inline Functions

inline bool VolumeImpl::operator == (const VolumeImpl &v) const
{
    // Do an element-wise comparison of the bounding boxes
    return v._box == _box;
}

inline bool VolumeImpl::operator != (const VolumeImpl &v) const
{
    // Do an element-wise comparison of the bounding boxes
    return !(v._box == _box);
}

#endif /* VolumeImpl.hh */
